<?php
// $Id:

/**
 * Private hook_help function, to reduce module overhead
 */
function _phpfreechat_help($section) {
  switch ($section) {
    case 'admin/modules#description':
      return t('Enables the creation of phpFreeChat channels on any page on the site.');
    case 'admin/help#phpfreechat':
      return t('<p>Enables the creation of phpFreeChat channels on any page on the site. ' .
               'You will need to enable phpFreeChat for any content types you would like a ' .
               'chat page on by going to: ' .
               l('/admin/settings/content-types', 'admin/settings/content-types') . 
               '.</p><p>phpFreeChat has the concept of chat channels. You can have as many channels ' .
               'as you want, and any chat box can connect to one or more channels on startup. ' .
               'By default a new channel will be created for each page, based on the page title. ' .
               'If you set the \'Default Channels\' setting below you can have each page join the ' .
               'same channel, creating a sitewide chatroom. ' .
               'In addition you can override the channel at the content-type or ' .
               'node level, which allows you to create a content-type channel or channels and/or ' .
               'set several pages to share the same channel or channels.</p>');
    case 'admin/settings/phpfreechat':
      return t('<p>Enables the creation of phpFreeChat channels on any page on the site.<br />' .
               'You will need to enable phpFreeChat for any content types you would like a ' .
               'chat page on by going to: ' .
               l('/admin/settings/content-types', 'admin/settings/content-types') . 
               '.</p><p>Please read the ' . l('phpFreeChat help pages', '/admin/help/phpfreechat') .
               ' to understand how to configure phpFreeChat channels.</p>' .
               '<p>Leaving any textbox blank will cause that setting to use the default value. ' .
               'Please note that some settings may not take effect until you clear your cookies.</p>');
  }
}

/**
 * Clears cached data from cookies
 */
function _phpfreechat_clear_cookies() {
  setcookie('phpfreechat', '', time() - 3600);
  setcookie('pfc_nickmarker', '', time() - 3600);
  setcookie('pfc_clock', '', time() - 3600);
  setcookie('pfc_minmax_status', '', time() - 3600);
  setcookie('pfc_current_text_color', '', time() - 3600);
  setcookie('pfc_showsmileys', '', time() - 3600);
  setcookie('pfc_showwhosonline', '', time() - 3600);
}

/**
 * Clears cached data from the configuration cache
 */
function _phpfreechat_clear_cache() {
  $cache_path = file_create_path('phpfreechat').'/private/cache/';
  if(is_dir($cache_path)) {
    $dir = opendir($cache_path);
    while (false != ($file = readdir($dir))) {
      if (($file != ".") && ($file != "..")) {
        unlink($cache_path.'/'.$file);
      }
    }
    closedir($dir);
  }
  drupal_set_message('phpFreeChat settings cache cleared');
}

function _phpfreechat_nuke() {
  file_delete(file_create_path('phpfreechat').'/chat/'.arg(2).'/messages.data');
  file_delete(file_create_path('phpfreechat').'/chat/'.arg(2).'/messages.index');
  touch(file_create_path('phpfreechat').'/chat/'.arg(2).'/messages.data');
  touch(file_create_path('phpfreechat').'/chat/'.arg(2).'/messages.index');
  drupal_set_message('Chat room contents cleared.');
  drupal_goto('');
}

/**
 * Private hook_settings function, to reduce module overhead
 */
function _phpfreechat_settings() {
  if (!empty($_POST['op'])) {
    _phpfreechat_clear_cache();
    _phpfreechat_clear_cookies();
  }
  if (!phpfreechat_check_install()) {
    form_set_error('op', phpfreechat_not_found());
  }

  $form['module'] = array(
    '#type' => 'fieldset',
    '#title' => t('phpFreeChat Module Settings'),
  );
  
    $form['module']['phpfreechat_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Default Title'),
      '#default_value' => variable_get('phpfreechat_title', ''),
      '#size' => 70,
      '#maxlength' => 128,
      '#description' => t('This is the default title (shown at the top of the chat box), which can be overriden at the content type or node level. If all of these are empty the node title will be used.'),
    );
    
    $form['module']['phpfreechat_channels'] = array(
      '#type' => 'textfield',
      '#title' => t('Default Channels'),
      '#default_value' => variable_get('phpfreechat_channels', ''),
      '#size' => 70,
      '#maxlength' => 128,
      '#description' => t('These are the default channels, which can be overriden at the content type or node level. ' .
          'You can enter multiple rooms to be joined separated by commas. ' .
          'Leaving this blank will auto-generate a string based on the title parameter.'),
    );

  $form['chat'] = array(
    '#type' => 'fieldset',
    '#title' => t('Chat Room Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

    $form['chat']['phpfreechat_auto_nick'] = array(
      '#type' => 'select',
      '#title' => t('Auto Nick'),
      '#default_value' => variable_get('phpfreechat_auto_nick', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Use site username as nick, rather than prompting the ' .
          'user to enter their own nick. (true value by default)'),
    );
    
    $form['chat']['phpfreechat_frozen_nick'] = array(
      '#type' => 'select',
      '#title' => t('Frozen Nick'),
      '#default_value' => variable_get('phpfreechat_frozen_nick', ''),
      '#options' => array('true' => 'true', '' => 'false'),
      '#description' => t('Setting this to true will forbid the user from ' .
          'changing their nickname later. (false value by default)'),
    );
    
    $form['chat']['phpfreechat_max_nick_len'] = array(
      '#type' => 'textfield',
      '#title' => t('Maximum Nick Length'),
      '#default_value' => variable_get('phpfreechat_max_nick_len', ''),
      '#description' => t('This is the maximum nickname length, a longer nickname ' .
          'is forbidden. (15 characters by default)'),
    );
    
    $form['chat']['phpfreechat_max_text_len'] = array(
      '#type' => 'textfield',
      '#title' => t('Maximum Message Length'),
      '#default_value' => variable_get('phpfreechat_max_text_len', ''),
      '#description' => t('This is the maximum message length, a longer message ' .
          'is forbidden. (250 characters by default)'),
    );
    
    $form['chat']['phpfreechat_max_msg'] = array(
      '#type' => 'textfield',
      '#title' => t('Message History Length'),
      '#default_value' => variable_get('phpfreechat_max_msg', ''),
      '#description' => t('This is the message history length. When a user ' .
          'connects, he can see the history. The number of messages s/he can ' .
          'see is defined by this (20 lines by default)'),
    );
    
    $form['chat']['phpfreechat_start_minimized'] = array(
      '#type' => 'select',
      '#title' => t('Start Minimized'),
      '#default_value' => variable_get('phpfreechat_start_minimized', ''),
      '#options' => array('true' => 'true', '' => 'false'),
      '#description' => t('Setting it to true will start the chat minimized. ' .
          '(false value by default)'),
    );
    
    $form['chat']['phpfreechat_height'] = array(
      '#type' => 'textfield',
      '#title' => t('Chat Height'),
      '#default_value' => variable_get('phpfreechat_height', ''),
      '#description' => t('Height of the chat area. (440px by default)'),
    );
    
    $form['chat']['phpfreechat_width'] = array(
      '#type' => 'textfield',
      '#title' => t('Chat Width'),
      '#default_value' => variable_get('phpfreechat_width', ''),
      '#description' => t('Width of the chat area. ' .
          '(empty by default, which means it\'s given by the CSS)'),
    );
    
    $form['chat']['phpfreechat_shownotice'] = array(
      '#type' => 'textfield',
      '#title' => t('Show Notice'),
      '#default_value' => variable_get('phpfreechat_shownotice', ''),
      '#description' => t('Setting this to 0 will disable nickname change ' .
          'notices and connect/disconnect notices. Setting it to 1 will only ' .
          'disable connect/disconnect notices. Setting it to 2 will show all ' .
          'notice types. (2 by default)'),
    );
    
    $form['chat']['phpfreechat_nickmarker'] = array(
      '#type' => 'select',
      '#title' => t('Nick Marker'),
      '#default_value' => variable_get('phpfreechat_nickmarker', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Setting this to false will disable nickname ' .
          'colorization. (true value by default)'),
    );
    
    $form['chat']['phpfreechat_clock'] = array(
      '#type' => 'select',
      '#title' => t('Clock'),
      '#default_value' => variable_get('phpfreechat_clock', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Setting this to false will hide the date/hour column. ' .
          '(true value by default)'),
    );
    
    $form['chat']['phpfreechat_openlinknewwindow'] = array(
      '#type' => 'select',
      '#title' => t('Open Links in New Window'),
      '#default_value' => variable_get('phpfreechat_openlinknewwindow', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Setting this to true will add target="_blank" into ' .
          'parsed links. This attribute is used to open the followed link in a ' .
          'new window. (true value by default)'),
    );
    
    $form['chat']['phpfreechat_showwhosonline'] = array(
      '#type' => 'select',
      '#title' => t('Show Who\'s Online'),
      '#default_value' => variable_get('phpfreechat_showwhosonline', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Used to show/hide online users at startup. ' .
          '(true value by default)'),
    );
    
    $form['chat']['phpfreechat_showsmileys'] = array(
      '#type' => 'select',
      '#title' => t('Show Smileys'),
      '#default_value' => variable_get('phpfreechat_showsmileys', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Used to show/hide the smiley list at startup. ' .
          '(true value by default)'),
    );
    
    $form['chat']['phpfreechat_btn_sh_whosonline'] = array(
      '#type' => 'select',
      '#title' => t('Show Who\'s Online Button'),
      '#default_value' => variable_get('phpfreechat_btn_sh_whosonline', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Used to display or not the showwhosonline button. ' .
          '(true value by default)'),
    );
    
    $form['chat']['phpfreechat_btn_sh_smileys'] = array(
      '#type' => 'select',
      '#title' => t('Show Smileys Button'),
      '#default_value' => variable_get('phpfreechat_btn_sh_smileys', ''),
      '#options' => array('' => 'true', 'false' => 'false'),
      '#description' => t('Used to display or not the showsmileys button. ' .
          '(true value by default)'),
    );
    
    $form['chat']['phpfreechat_theme'] = array(
      '#type' => 'textfield',
      '#title' => t('Theme'),
      '#default_value' => variable_get('phpfreechat_theme', ''),
      '#description' => t('This parameter specifies which theme this chat will ' .
          'use. ("default" when empty)'),
    );
  
    $form['chat']['phpfreechat_language'] = array(
      '#type' => 'textfield',
      '#title' => t('Language'),
      '#default_value' => variable_get('phpfreechat_language', ''),
      '#description' => t('Used to specify the chat labels language. ' .
          'Accepted values are i18n/* sub directories names. ' .
          '(by default this is the local server language)'),
    );

  $form['system'] = array(
    '#type' => 'fieldset',
    '#title' => t('Performance & Output Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

    $form['system']['phpfreechat_container_type'] = array(
      '#type' => 'textfield',
      '#title' => t('Container Type'),
      '#default_value' => variable_get('phpfreechat_container_type', ''),
      '#description' => t('Used to specify the used container. For the moment, ' .
          'the file container is the best supported. A \'Memory\' ' .
          'container, which offers much greater performance has also been ' .
          'written, but is less well tested. In the future several ' .
          'containers (IRC, MSN, Jabber...) will surely be available. ' .
          '(\'File\' by default)'),
    );
  
    $form['system']['phpfreechat_container_cfg_sm_type'] = array(
      '#type' => 'textfield',
      '#title' => t('Container Storage Method'),
      '#default_value' => variable_get('phpfreechat_container_cfg_sm_type', ''),
      '#description' => t('Sets the storage method for the selected container ' .
          'type. Currently this only applies for the \'Memory\' container - ' .
          'possible values are: Eaccelerator, File, Shmop, Sharedance (tested), ' .
          'Systemv, Mmcache, Memcache, Apc, Apachenote, Sqlite (not tested) .' .
          '(\'auto\' by default)'),
    );
  
    $form['system']['phpfreechat_refresh_delay'] = array(
      '#type' => 'textfield',
      '#title' => t('Refresh Delay'),
      '#default_value' => variable_get('phpfreechat_refresh_delay', ''),
      '#description' => t('This is the time to wait (in milliseconds) between' .
          ' two refreshes. A refresh is a HTTP request which asks the server ' .
          'if there are new messages to display. If there are no new messages, ' .
          'then a empty HTTP response is returned. (5000 by default)'),
    );
    
    $form['system']['phpfreechat_timeout'] = array(
      '#type' => 'textfield',
      '#title' => t('Timeout'),
      '#default_value' => variable_get('phpfreechat_timeout', ''),
      '#description' => t('This is the time of inactivity (in milliseconds)' .
          'to wait before disconnecting the user. (20000 by default)'),
    );
    
    $form['system']['phpfreechat_connect_at_startup'] = array(
      '#type' => 'select',
      '#title' => t('Connect At Startup'),
      '#default_value' => variable_get('phpfreechat_connect_at_startup', ''),
      '#options' => array('true' => 'true', 'false' => 'false'),
      '#description' => t('Setting this to false will oblige user to click ' .
          'on the connect button if s/he wants to chat. (true value by ' .
          'default, which means when the chat web page is open, a connection ' .
          'to the chat is automaticaly performed)'),
    );
    
    $form['system']['phpfreechat_output_encoding'] = array(
      '#type' => 'textfield',
      '#title' => t('Output Encoding'),
      '#default_value' => variable_get('phpfreechat_output_encoding', ''),
      '#description' => t('Useful to set a specific encoding for chat labels. ' .
          'This is really useful when the Web page embedding the chat is ' .
          'not UTF-8 encoded. This parameter should be the same as the chat ' .
          'web page. (UTF-8 by default)'),
    );
    
    $form['system']['phpfreechat_prefix'] = array(
      '#type' => 'textfield',
      '#title' => t('Prefix'),
      '#default_value' => variable_get('phpfreechat_prefix', ''),
      '#description' => t('Used to prefix all javascript functions, CSS ' .
          'id\'s and classes in order to avoid namespace clashes. ' .
          '("pfc_" by default)'),
    );
  
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

    $form['advanced']['phpfreechat_params'] = array(
      '#type' => 'select',
      '#title' => t('Debug Parameters'),
      '#default_value' => variable_get('phpfreechat_params', ''),
      '#options' => array('true' => 'true', '' => 'false'),
      '#description' => t('For developers, setting this to true will list ' .
          'the chat parameters, session and cookies. (false value by default)'),
    );

    $form['advanced']['phpfreechat_debug'] = array(
      '#type' => 'select',
      '#title' => t('Debug Console'),
      '#default_value' => variable_get('phpfreechat_debug', ''),
      '#options' => array('true' => 'true', '' => 'false'),
      '#description' => t('For developers, setting this to true will activate ' .
          'logging and debuging console. (false value by default)'),
    );
    
    $form['advanced']['phpfreechat_debugxajax'] = array(
      '#type' => 'select',
      '#title' => t('Debugxajax'),
      '#default_value' => variable_get('phpfreechat_debugxajax', ''),
      '#options' => array('true' => 'true', '' => 'false'),
      '#description' => t('For developers, setting it to true will activate ' .
          'xajax js popup used to trace communication between client and ' .
          'server during ajax requests. Useful to discover hidden bugs like ' .
          'php warnings on a ajax request. (false value by default)'),
    );
    $form['advanced']['phpfreechat_server_script_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Server Script Path'),
      '#default_value' => variable_get('phpfreechat_server_script_path', ''),
      '#description' => t('Used to specify the script which will handle ' .
          'asynchronous requests. (by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_server_script_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Server Script URL'),
      '#default_value' => variable_get('phpfreechat_server_script_url', ''),
      '#description' => t('Used to specify the script which will handle ' .
          'asynchronous requests. (by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_client_script_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Client Script Path'),
      '#default_value' => variable_get('phpfreechat_client_script_path', ''),
      '#description' => t('Used to specify the script path which display the ' .
          'chat and load every JavaScript libraries (the client side). ' .
          'Useful when the php configuration is uncommon, this option can ' .
          'be used to force the automatic detection process. client_script_url ' .
          'must point to the client script browable url (useful when using URL ' .
          'rewriting). (by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_client_script_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Client Script URL'),
      '#default_value' => variable_get('phpfreechat_client_script_url', ''),
      '#description' => t('Used to specify the script path which display the ' .
          'chat and load every JavaScript libraries (the client side). ' .
          'Useful when the php configuration is uncommon, this option can be ' .
          'used to force the automatic detection process. client_script_url ' .
          'must point to the client script browable URL (useful when using URL ' .
          'rewriting). (by default this parameter is auto-detected)'),
    );
  
    $form['advanced']['phpfreechat_themeurl'] = array(
      '#type' => 'textfield',
      '#title' => t('Themeurl'),
      '#default_value' => variable_get('phpfreechat_themeurl', ''),
      '#description' => t('Used to specify the theme root URL (useful for URL ' .
          'rewriting) (by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_themepath'] = array(
      '#type' => 'textfield',
      '#title' => t('Themepath'),
      '#default_value' => variable_get('phpfreechat_themepath', ''),
      '#description' => t('The absolute theme directory path. Sub directories ' .
          'must contains themes directories. ' .
          '(dirname(__FILE__)."/../themes" by default)'),
    );
    
    $form['advanced']['phpfreechat_jspath'] = array(
      '#type' => 'textfield',
      '#title' => t('Jspath'),
      '#default_value' => variable_get('phpfreechat_jspath', ''),
      '#description' => t('Specify the javascript libraries path (md5, ' .
          'cookie ...). (dirname(__FILE__)."/../lib/javascript" by default)'),
    );
    
    $form['advanced']['phpfreechat_smartypath'] = array(
      '#type' => 'textfield',
      '#title' => t('Smartypath'),
      '#default_value' => variable_get('phpfreechat_smartypath', ''),
      '#description' => t('Specify the Smarty path. It\'s useful if you ' .
          'already have a Smarty library installed and you want to save bytes ' .
          'on your server. ' .
          '(dirname(__FILE__)."/../lib/Smarty-x.x.x" by default)'),
    );
    
    $form['advanced']['phpfreechat_xajaxpath'] = array(
      '#type' => 'textfield',
      '#title' => t('Xajaxpath'),
      '#default_value' => variable_get('phpfreechat_xajaxpath', ''),
      '#description' => t('Specify the xajax path. It\'s useful if you already ' .
          'have an xajax library installed and you want to save bytes on your ' .
          'server. (dirname(__FILE__)."/../lib/xajax_x.x" by default);'),
    );
    
    $form['advanced']['phpfreechat_data_private_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Data Private Path'),
      '#default_value' => variable_get('phpfreechat_data_private_path', ''),
      '#description' => t('Used by the filesystem container to store chat ' .
          'data. Used by smarty to store compiled templates. ' .
          'Tip: you can optimize your chat performance, by pointing this to' .
          'a RAM filesystem mount. ' .
          '(by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_data_public_path'] = array(
      '#type' => 'textfield',
      '#title' => t('Data Public Path'),
      '#default_value' => variable_get('phpfreechat_data_public_path', ''),
      '#description' => t('This path must be reachable by your web server. ' .
          'JS compiled scripts will be stored here. data_public_url parameter ' .
          'can be specified to help when URL rewriting is used. ' .
          '(by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_data_public_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Data Public URL'),
      '#default_value' => variable_get('phpfreechat_data_public_url', ''),
      '#description' => t('This path must be reachable by your web server. ' .
          'JS compiled scripts will be stored here. data_public_url parameter ' .
          'can be specified to help when URL rewriting is used. ' .
          '(by default this parameter is auto-detected)'),
    );
    
    $form['advanced']['phpfreechat_serverid'] = array(
      '#type' => 'textfield',
      '#title' => t('Serverid'),
      '#default_value' => variable_get('phpfreechat_serverid', ''),
      '#description' => t('This is the chat server id, ' .
          '(comparable to the server host in IRC) used to identify the server. ' .
          'Is you want to share channels between different sites you can use this ' .
          'to force an identical serverid. ' .
          '(by default this parameter is auto-detected)'),
    );
    
  return $form;
}